'use strict'
const fs = require('hexo-fs')

hexo.extend.filter.register('after_render:html', function (str) {

  const idReg = /<div +id *= *["']disqus_thread["'] *>/
  const srcReg = /s.src *= *['"]https:\/\/.+\.disqus\.com\/embed\.js['"];?/
  const insertReg = /\(d\.head *\|\| *d\.body\)\.appendChild\(s\);?/

  if (idReg.test(str) && srcReg.test(str) && insertReg.test(str)) {
    // 将disqus_thread转化为disqus_proxy_thread
    str = str.replace(idReg, '<div id="disqus_proxy_thread">')

    // 插入CND和外部资源
    const cdn = `
      <script src="//cdn.bootcss.com/react/16.0.0/umd/react.production.min.js"></script>
      <script src="//cdn.bootcss.com/react-dom/16.0.0/umd/react-dom.production.min.js"></script>
      <script src="//cdn.bootcss.com/moment.js/2.18.1/moment.min.js"></script>
      <script src="//cdn.bootcss.com/es6-promise/4.1.1/es6-promise.auto.min.js"></script>
      <link rel="stylesheet" href="//cdn.bootcss.com/font-awesome/4.7.0/css/font-awesome.min.css"/>
      <link rel="stylesheet" href="/styles/hexo-disqus-proxy.css"/>
      `
    str = str.replace(/<head>/, '<head>' + cdn)

    // hexo-disqus-proxy-primary主脚本
    const script = `<script src="/scripts/hexo-disqus-proxy-primary.js" async></script>`
    const n = str.lastIndexOf('</body>')
    str = str.substring(0, n) + str.substring(n).replace('</body>', script + '</body>')

    // 删除原有的disqus评论脚本
    str = str.replace(srcReg, '')
    str = str.replace(insertReg, '')
  }

  return str
})

hexo.extend.generator.register('assets', function (locals) {
  const config = hexo.config.disqus_proxy
  const asset = [
    {
      path: 'scripts/hexo-disqus-proxy-primary.js',
      data: function () {
        return fs.createReadStream('node_modules/hexo-disqus-proxy/lib/hexo-disqus-proxy-primary.js')
      }
    },
    {
      path: 'scripts/disqus-proxy.chunk.0.js',
      data: function () {
        return fs.createReadStream('node_modules/hexo-disqus-proxy/lib/disqus-proxy.chunk.0.js')
      }
    },
    {
      path: 'styles/hexo-disqus-proxy.css',
      data: function () {
        return fs.createReadStream('node_modules/hexo-disqus-proxy/lib/styles/hexo-disqus-proxy.css')
      }
    }
  ]
  if (!config.admin_avatar) asset.push({
    path: 'avatars/admin-avatar.jpg',
    data: function () {
      return fs.createReadStream('node_modules/hexo-disqus-proxy/lib/avatars/admin-avatar.jpg')
    }
  })
  if (!config.default_avatar) asset.push({
    path: 'avatars/default-avatar.png',
    data: function () {
      return fs.createReadStream('node_modules/hexo-disqus-proxy/lib/avatars/default-avatar.png')
    }
  })
  return asset
})

hexo.extend.filter.register('template_locals', function (locals) {

  if (!locals.page) return locals

  const config = hexo.config.disqus_proxy

  const script = `
      <script>
        window.disqusProxy={
          shortname: '${config.shortname}',
          server: '${config.host}',
          port: ${config.port},
          defaultAvatar: '${config.default_avatar ? config.default_avatar : "/avatars/default-avatar.png"}',
          adminAvatar: '${config.admin_avatar ? config.admin_avatar : "/avatars/admin-avatar.jpg"}',
          identifier: '${locals.page.path}',
        };
        window.disqus_config = function () {
          this.page.url = window.location.href;
          this.page.identifier = window.disqusProxy.identifier;
        };
      </script>`
  locals.page.content = locals.page.content + script

  return locals
})
